
/**
 * @imports
 */
import cssSync from '../../css/readSync.js';
import cssAsync from '../../css/readAsync.js';
import TransformRule from '../../Css/TransformRule.js';
import Destination from './Destination.js';
import play from '../../ani/play.js';

/**
 * Translates an element to a reference point in the UI.
 */
export default class Translate extends Destination {

    /**
     * @inheritdoc
     */
    constructor(el, reference, options = {}) {
        super(...arguments);
        this.translateFormatted = {
            from: new TransformRule({translate: [],}),
            to: new TransformRule({translate: [],}),
            current: {
                from: new TransformRule({translate: [],}),
                to: new TransformRule({translate: [],}),
                progress: {},
            },
        };
        // -----------
        var activeTransform = cssSync(this.el, 'transform');
        ['x', 'y'].forEach((axis, i) => {
            if (axis in this.coords) {
                this.translateFormatted.from.translate[i] = 0;
                this.translateFormatted.to.translate[i] = this.coords[axis] + activeTransform.translate[i];
                // ----------------
                this.translateFormatted.current.from.translate[i] = activeTransform.translate[i];
                this.translateFormatted.current.to.translate[i] = this.coords[axis];
                // ----------------
                this.translateFormatted.current.progress[axis] = Math.abs(activeTransform.translate[i] / (this.coords[axis] + activeTransform.translate[i]));
            } else {
                this.translateFormatted.from.translate[i] = activeTransform.translate[i];
                this.translateFormatted.to.translate[i] = activeTransform.translate[i];
                // ----------------
                this.translateFormatted.current.from.translate[i] = activeTransform.translate[i];
                this.translateFormatted.current.to.translate[i] = activeTransform.translate[i];
                // ----------------
                this.translateFormatted.current.progress[axis] = 0;
            }
        });
    }
    
    /**
     * Plays the Translation.
     * 
     * @param object    timing
     * 
     * @return Promise
     */
    play(timing = {}) {
        return play(this.el, [
            {transform: this.translateFormatted.current.from},
            {transform: this.translateFormatted.to},
        ], timing);
    }

    /**
     * Sets the Translation by CSS.
     * 
     * @return Promise
     */
    css() {
        return cssAsync(this.el, {transform: this.translateFormatted.to});
    }
};
